/*
 * ExtendedFileStatistics.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;


/**
 * <p>
 * This is the extended version of FileStatistics which supports some advanced operations including directory
 * processing and printing of the report. The reason to separate this out is that we may not need to rely on the
 * directory traversing algorithm incorporated here (which is recursive).
 * </p>
 * 
 * <p>
 * This class shares the same configuration logic as FileStatistics. The constructor simply propagates the namespace to
 * the super class.
 * </p>
 * 
 * <p>
 * All the operations are delegated. This class does not have any states and is fully dependent on FileStatistics
 * threading level.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class ExtendedFileStatistics extends FileStatistics {
    /**
     * <p>
     * Default constructor. Simply propagates to super constructor.
     * </p>
     */
    public ExtendedFileStatistics() {
        super();
    }

    /**
     * <p>
     * Constructor to load configuration from given namespace. Simply propagates to super constructor.
     * </p>
     *
     * @param namespace the namespace to load from
     *
     * @throws NullPointerException if namespace is null
     * @throws IllegalArgumentException if namespace is empty
     * @throws ConfigurationException if configuration error occurs
     */
    public ExtendedFileStatistics(String namespace) throws ConfigurationException {
        super(namespace);
    }

    /**
     * <p>
     * Processes a directory in non-recursive mode. This is equivalent to calling processDirectory(directory, false).
     * </p>
     *
     * @param directory the directory to process
     *
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if directory is not a valid directory
     * @throws IOException if I/O operation fails
     * @throws FileStatisticsException if any error occurs in processing the directory
     */
    public void processDirectory(File directory) throws IOException, FileStatisticsException {
        processDirectory(directory, false);
    }

    /**
     * <p>
     * Processes a directory in the given mode. If it's non-recursive only the files in the directory are processed.
     * Otherwise the files in all the sub-directories are processed.
     * </p>
     *
     * @param directory the directory to process
     * @param recursive whether to process the directory recursively
     *
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if directory is not a valid directory
     * @throws IOException if I/O operation fails
     * @throws FileStatisticsException if any error occurs in processing the directory
     */
    public void processDirectory(File directory, boolean recursive)
        throws IOException, FileStatisticsException
    {
        if (directory == null) {
            throw new NullPointerException("directory is null");
        }
        if (!directory.isDirectory()) {
            throw new IllegalArgumentException("directory is not a valid directory");
        }

        File[] files = directory.listFiles();

        for (int i = 0; i < files.length; i++) {
            if (files[i].isDirectory()) {
                // if it's a directory, recursively process the directory if recursive flag is set
                if (recursive) {
                    processDirectory(files[i], recursive);
                }
            } else {
                // otherwise, process the file as normal
                processFile(files[i]);
            }
        }
    }

    /**
     * <p>
     * Prints the report to console.
     * </p>
     *
     * @throws StatsReportingException if error occurs in generating report
     */
    public void printReport() throws StatsReportingException {
        System.out.print(getReporting().generateReport());
    }

    /**
     * <p>
     * Prints the report to the given file.
     * </p>
     *
     * @param file the file to write to
     *
     * @throws NullPointerException if file is null
     * @throws IOException if I/O operation fails
     * @throws StatsReportingException if error occurs in generating report
     */
    public void printReport(File file) throws IOException, StatsReportingException {
        if (file == null) {
            throw new NullPointerException("file is null");
        }

        Writer out = null;

        try {
            // create a BufferedWriter to write a String
            out = new BufferedWriter(new FileWriter(file));

            // write the report to the file
            out.write(getReporting().generateReport());
        } finally {
            close(out);
        }
    }

    /**
     * <p>
     * Try to close the writer object if it is not null. Returns silently if any I/O error occurs.
     * </p>
     *
     * @param writer the writer object.
     */
    private static void close(Writer writer) {
        if (writer != null) {
            try {
                writer.close();
            } catch (IOException e) {}
        }
    }
}
