/*
 * FileCategoryMatcher.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.FileMatcher;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;


/**
 * <p>
 * FileCategoryMatcher is an abstract class that handles file category (text/binary) oriented matching logic. The
 * purpose to still create subclasses for such minor functionality is about the convenience in configuration.
 * </p>
 * 
 * <p>
 * This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class FileCategoryMatcher implements FileMatcher {
    /**
     * <p>
     * The number of bytes to check at the beginning of the file to determine if the file is binary.
     * </p>
     */
    private static final int BYTE_TO_READ = 1024;

    /**
     * <p>
     * Default protected constructor to be accessed by subclass. Does nothing.
     * </p>
     */
    protected FileCategoryMatcher() {
    }

    /**
     * <p>
     * Checks if the file is binary. Note that an empty file is treated as a text file (non-binary). This method is
     * exposed as public static because it could potentially be used by other custom matchers.
     * </p>
     * 
     * <p>
     * This method does:
     * 
     * <ul>
     * <li>
     * Open a binary stream and check at most 1024 bytes.
     * </li>
     * <li>
     * If a byte is not within [32, 127] and is not tab, carriage return or new line, it's a binary file; otherwise
     * it's a text file.
     * </li>
     * <li>
     * In any case the stream should be closed.
     * </li>
     * </ul>
     * </p>
     *
     * @param file the file to check
     *
     * @return whether the file is binary
     *
     * @throws NullPointerException if file is null
     * @throws IOException if I/O operation fails
     */
    public static boolean isBinary(File file) throws IOException {
        if (file == null) {
            throw new NullPointerException("file is null");
        }

        InputStream in = null;

        try {
            // open a buffered binary stream to ensure efficiency
            in = new BufferedInputStream(new FileInputStream(file));

            int b = -1;
            int count = 0;

            // check if the bytes fall into the binary range
            while ((count < BYTE_TO_READ) && ((b = in.read()) != -1)) {
                if (((b < 32) || (b > 127)) && !((b == '\t') || (b == '\r') || (b == '\n'))) {
                    return true;
                }
                count++;
            }

            // if the bytes do not fall into the binary range, it must be a text file
            return false;
        } finally {
            close(in);
        }
    }

    /**
     * <p>
     * Determines if the file can be processed by the processor that this matcher is associated with.
     * </p>
     * 
     * <p>
     * This method gets the file category and passes it to the abstract match() method.
     * </p>
     *
     * @param file the file to match
     *
     * @return whether the file matches
     *
     * @throws NullPointerException if file is null
     * @throws IOException if I/O operation fails
     */
    public boolean match(File file) throws IOException {
        return match(isBinary(file));
    }

    /**
     * <p>
     * Determines whether the file matches based on the given file category. Implementations should define the logic to
     * match the file category.
     * </p>
     *
     * @param isBinary whether the file is binary
     *
     * @return whether the file category matches
     */
    protected abstract boolean match(boolean isBinary);

    /**
     * <p>
     * Try to close the InputStream object if it is not null. Returns silently if any I/O error occurs.
     * </p>
     *
     * @param input the InputStream object.
     */
    private static void close(InputStream input) {
        if (input != null) {
            try {
                input.close();
            } catch (IOException e) {}
        }
    }
}
