/*
 * FileTypeMatcher.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.FileMatchingException;
import com.topcoder.file.type.FileType;
import com.topcoder.file.type.MagicNumbers;

import java.io.File;
import java.io.IOException;


/**
 * <p>
 * FileTypeMatcher is an abstract class that handles file type oriented matching logic. File type is resolved by the
 * Magic Numbers component. The configuration of Magic Numbers should be done via the configuration file.
 * </p>
 * 
 * <p>
 * This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class FileTypeMatcher implements FileMatcher {
    /**
     * <p>
     * Default protected constructor to be accessed by subclass. Does nothing.
     * </p>
     */
    protected FileTypeMatcher() {
    }

    /**
     * <p>
     * Determines if the file can be processed by the processor that this matcher is associated with.
     * </p>
     * 
     * <p>
     * This method just obtains the file type using the Magic Numbers component and passes it to the abstract match()
     * method. The proper configuration of Magic Numbers is not within the responsibility of this class. Any issue
     * around this is translated into FileMatchingException. If the file does not have a corresponding file type,
     * false will be returned.
     * </p>
     *
     * @param file the file to match
     *
     * @return whether the file matches
     *
     * @throws NullPointerException if file is null
     * @throws IOException if I/O operation fails
     * @throws FileMatchingException if error occurs in file matching
     */
    public boolean match(File file) throws IOException, FileMatchingException {
        if (file == null) {
            throw new NullPointerException("file is null");
        }

        // create the MagicNumbers instance from configuration file
        MagicNumbers mn = null;

        try {
            mn = new MagicNumbers();
        } catch (Exception e) {
            // if any problem occurs in configuring the MagicNumbers,
            // wrap it as FileMatchingException
            throw new FileMatchingException("Fails to create MagicNumbers instance", e);
        }

        // determine if the file matches based on its file type
        FileType type = mn.determineFileType(file);

        if (type == null) {
            return false;
        } else {
            return match(type);
        }
    }

    /**
     * <p>
     * Determines whether the file matches based on the given file type. Implementations should define the logic to
     * match the file type.
     * </p>
     * 
     * <p>
     * This method assumes that the type argument will never be null. I/O errors should not occur here since the
     * matching logic is file type oriented.
     * </p>
     *
     * @param type the file type to match
     *
     * @return whether the file type matches
     *
     * @throws FileMatchingException if error occurs in file type matching
     */
    protected abstract boolean match(FileType type) throws FileMatchingException;
}
