/*
 * CStyleLineCounter.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.processor.linecounter;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.processor.FileProcessorBase;

import com.topcoder.util.config.Property;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;

import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * CStyleLineCounter counts the number of lines for C style source files. This implementation can handle C, C++, or
 * Java. It only takes braces ('{') and semi-colons (';') into account so it would not be very accurate in most cases.
 * Comments and string literals can be handled. It supports a single metric of "Line Count".
 * </p>
 * 
 * <p>
 * It delegates configuration to base class without any addition. The implementation still attempts to process the file
 * even if it is not a C style source file.
 * </p>
 * 
 * <p>
 * This class is stateless while its base class is thread-safe, thus it is thread-safe too.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class CStyleLineCounter extends FileProcessorBase {
    /**
     * <p>
     * The name of the metric used for this processor.
     * </p>
     */
    public static final String NAME = "Line Count";

    /**
     * <p>
     * The display name of the metric used for this processor.
     * </p>
     */
    public static final String DISPLAY_NAME = "line";

    /**
     * <p>
     * The description of the metric used for this processor.
     * </p>
     */
    private static final String DESCRIPTION = "Number of lines in C style source files";

    /**
     * <p>
     * The single metric supported by this processor. It is statically initialized and never changed.
     * </p>
     */
    private static final Metric METRIC = new Metric(NAME, DISPLAY_NAME, DESCRIPTION);

    /**
     * <p>
     * Constructor with the given alias and file matcher. Just propagates to super constructor.
     * </p>
     *
     * @param alias the alias for the file processor
     * @param matcher the file matcher associated with the file processor
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty
     */
    public CStyleLineCounter(String alias, FileMatcher matcher) {
        super(alias, matcher);
    }

    /**
     * <p>
     * This constructor loads configuration from the property. Just propagates to super constructor.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if any error occurs in configuring the processor
     */
    public CStyleLineCounter(Property property) throws ConfigurationException {
        super(property);
    }

    /**
     * <p>
     * Returns a list containing single Metric composed of the NAME, DISPLAY_NAME and a description.
     * </p>
     *
     * @return a list of metrics the processor supports
     */
    public List getSupportedMetrics() {
        List metrics = new ArrayList();

        // add single metric to the list
        metrics.add(METRIC);

        return metrics;
    }

    /**
     * <p>
     * Counts the number of lines in the given C style source file. The returned StatsResult will include the line
     * count under the single metric of "Line Count".
     * </p>
     * 
     * <p>
     * An empty file should have a line count of zero.
     * </p>
     *
     * @param file the file to process
     *
     * @return the line count result
     *
     * @throws NullPointerException if file is null
     * @throws IOException if I/O operation fails
     */
    public StatsResult process(File file) throws IOException {
        if (file == null) {
            throw new NullPointerException("file is null");
        }

        // read the lines using a BufferedReader
        BufferedReader in = null;
        SimpleStateMachine machine = new SimpleStateMachine();

        try {
            in = new BufferedReader(new FileReader(file));

            String line = null;

            // process each line using the SimpleStateMachine
            while ((line = in.readLine()) != null) {
                machine.process(line);
            }
        } finally {
            close(in);
        }

        StatsResult result = new StatsResult();

        // include the line count under the single metric
        result.setStat(METRIC, machine.getLines());

        return result;
    }

    /**
     * <p>
     * Try to close the reader object if it is not null. Returns silently if any I/O error occurs.
     * </p>
     *
     * @param reader the reader object.
     */
    private static void close(Reader reader) {
        if (reader != null) {
            try {
                reader.close();
            } catch (IOException e) {}
        }
    }

    /**
     * <p>
     * This is a private static inner class to CStyleLineCounter that implements a state machine that can process the
     * source code. Refer to Component Specification for the state machine description.
     * </p>
     * 
     * <p>
     * This class is stateful and not thread-safe. Each instance only serves a single process.
     * </p>
     */
    private static class SimpleStateMachine {
        /**
         * <p>
         * Represents the state not in comment or literal.
         * </p>
         */
        private static final int STATE_START = 1;

        /**
         * <p>
         * Represents the state after a single slash. (Input on slash would enter line comment state; input on star
         * would enter block state.)
         * </p>
         */
        private static final int STATE_SLASH = 2;

        /**
         * <p>
         * Represents the state of a star in a block comment. (Input on slash would enter start state.)
         * </p>
         */
        private static final int STATE_STAR = 3;

        /**
         * <p>
         * Represents the state of back slash in single quote literal (next character is escaped).
         * </p>
         */
        private static final int STATE_SQ_BACKSLASH = 4;

        /**
         * <p>
         * Represents the state of back slash in double quote literal (next character is escaped).
         * </p>
         */
        private static final int STATE_DQ_BACKSLASH = 5;

        /**
         * <p>
         * Represents the state in single quote literal.
         * </p>
         */
        private static final int STATE_SINGLE_QUOTE = 6;

        /**
         * <p>
         * Represents the state in double quote literal.
         * </p>
         */
        private static final int STATE_DOUBLE_QUOTE = 7;

        /**
         * <p>
         * Represents the state in block comment.
         * </p>
         */
        private static final int STATE_BLOCK_COMMENT = 8;

        /**
         * <p>
         * Represents the state in line comment.
         * </p>
         */
        private static final int STATE_LINE_COMMENT = 9;

        /**
         * <p>
         * The slash character for block comment or line comment.
         * </p>
         */
        private static final char SLASH_CHAR = '/';

        /**
         * <p>
         * The semicolon character to increment line count.
         * </p>
         */
        private static final char SEMICOLON_CHAR = ';';

        /**
         * <p>
         * The opening brace character to increment line count.
         * </p>
         */
        private static final char OPEN_BRACE_CHAR = '{';

        /**
         * <p>
         * The star character to open or close block comment.
         * </p>
         */
        private static final char STAR_CHAR = '*';

        /**
         * <p>
         * The backslash character to escape non-printable character.
         * </p>
         */
        private static final char BACK_SLASH_CHAR = '\\';

        /**
         * <p>
         * The single quote character to start or end a character literal.
         * </p>
         */
        private static final char SINGLE_QUOTE_CHAR = '\'';

        /**
         * <p>
         * The double quote character to start or end a string literal.
         * </p>
         */
        private static final char DOUBLE_QUOTE_CHAR = '\"';

        /**
         * <p>
         * The line feed character to end line comment.
         * </p>
         */
        private static final char LINE_FEED_CHAR = '\n';

        /**
         * <p>
         * The current state of this state machine. References one of the defined constants.
         * </p>
         */
        private int state = STATE_START;

        /**
         * <p>
         * The current line count. Initialized to 0 and increments whenever necessary.
         * </p>
         */
        private long lines = 0;

        /**
         * <p>
         * Default constructor. The current state is the start state and line count is zero.
         * </p>
         */
        public SimpleStateMachine() {
        }

        /**
         * <p>
         * Processes a line. It processes each character as well as a line feed at the end. The state and line count is
         * updated after processing.
         * </p>
         *
         * @param line the line to process, not including the line feed
         *
         * @throws NullPointerException if line is null
         */
        public void process(String line) {
            if (line == null) {
                throw new NullPointerException("line is null");
            }

            // process each character of the line
            for (int i = 0; i < line.length(); i++) {
                process(line.charAt(i));
            }

            // process the line feed at the end
            process(LINE_FEED_CHAR);
        }

        /**
         * <p>
         * Processes a character from input. Refer to Component Specification for algorithm. The state and line count
         * is updated after processing.
         * </p>
         *
         * @param ch the character to process
         */
        private void process(char ch) {
            if (state == STATE_START) {
                // process from start - can enter comment or string literal
                if (ch == SLASH_CHAR) {
                    state = STATE_SLASH;
                } else if (ch == SINGLE_QUOTE_CHAR) {
                    state = STATE_SINGLE_QUOTE;
                } else if (ch == DOUBLE_QUOTE_CHAR) {
                    state = STATE_DOUBLE_QUOTE;
                } else if ((ch == SEMICOLON_CHAR) || (ch == OPEN_BRACE_CHAR)) {
                    // increment line count when opening brace or semicolon is encountered
                    lines++;
                }
            } else if (state == STATE_SLASH) {
                // process from slash - can enter block comment or line comment
                if (ch == STAR_CHAR) {
                    state = STATE_BLOCK_COMMENT;
                } else if (ch == SLASH_CHAR) {
                    state = STATE_LINE_COMMENT;
                } else {
                    state = STATE_START;
                }
            } else if (state == STATE_STAR) {
                // process from star - can leave the block comment and enter start
                if (ch == SLASH_CHAR) {
                    state = STATE_START;
                } else if (ch != STAR_CHAR) {
                    state = STATE_BLOCK_COMMENT;
                }
            } else if (state == STATE_SQ_BACKSLASH) {
                // process from backslash in single quote - next character is escaped
                state = STATE_SINGLE_QUOTE;
            } else if (state == STATE_DQ_BACKSLASH) {
                // process from backslash in double quote - next character is escaped
                state = STATE_DOUBLE_QUOTE;
            } else if (state == STATE_SINGLE_QUOTE) {
                // process from single quote - can enter escaped character or
                // leave the string literal and enter start
                if (ch == BACK_SLASH_CHAR) {
                    state = STATE_SQ_BACKSLASH;
                } else if (ch == SINGLE_QUOTE_CHAR) {
                    state = STATE_START;
                }
            } else if (state == STATE_DOUBLE_QUOTE) {
                // process from double quote - can enter escaped character or
                // leave the string literal and enter start
                if (ch == BACK_SLASH_CHAR) {
                    state = STATE_DQ_BACKSLASH;
                } else if (ch == DOUBLE_QUOTE_CHAR) {
                    state = STATE_START;
                }
            } else if (state == STATE_LINE_COMMENT) {
                // process from line comment - can leave the line comment
                // and enter start for next line
                if (ch == LINE_FEED_CHAR) {
                    state = STATE_START;
                }
            } else if (state == STATE_BLOCK_COMMENT) {
                // process from block comment - can enter star to close the block comment
                if (ch == STAR_CHAR) {
                    state = STATE_STAR;
                }
            }
        }

        /**
         * <p>
         * Gets the current line count.
         * </p>
         *
         * @return the current line count
         */
        public long getLines() {
            return lines;
        }
    }
}
