/*
 * DemoTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.BinaryFileMatcher;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.matcher.MimeMatcher;
import com.topcoder.file.statistics.matcher.RegexMatcher;
import com.topcoder.file.statistics.matcher.TextFileMatcher;
import com.topcoder.file.statistics.matcher.TypeNameMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.BasicStatsReporting;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import java.util.List;
import java.util.regex.Pattern;


/**
 * A simple demonstration of the File Statistics component. First, the File Statistics is configured on the processors,
 * reporting implementation and matchall flag. It then can be used to process files and directories. Reports can be
 * generated based on the processing result.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class DemoTest extends TestCase {
    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(DemoTest.class);

        return suite;
    }

    /**
     * Shows how the File Statistics is configured on the processors, reporting implementation and matchall flag. In
     * addition, a matcher has to be associated with each processor. Below shows all the supported matchers,
     * processors and reporting implementations in this release.
     */
    public void testConfigure() {
        // create a file statistics instance
        ExtendedFileStatistics statistics = new ExtendedFileStatistics();

        // create file matchers
        FileMatcher matcher1 = new AnyFileMatcher();
        FileMatcher matcher2 = new ExtensionMatcher("txt");
        FileMatcher matcher3 = new RegexMatcher(Pattern.compile("pattern"));
        FileMatcher matcher4 = new TextFileMatcher();
        FileMatcher matcher5 = new BinaryFileMatcher();
        FileMatcher matcher6 = new TypeNameMatcher("Java");
        FileMatcher matcher7 = new MimeMatcher("text/plain");

        // create file processors
        FileProcessor processor1 = new TextLineCounter("Text File", matcher1);
        FileProcessor processor2 = new CStyleLineCounter("Source File", matcher6);

        // create stats reporting implementation
        StatsReporting reporting1 = new BasicStatsReporting();
        StatsReporting reporting2 = new XmlStatsReporting(new File("test_files/stylesheet.xsl"));

        // manipulate file processors
        statistics.addFileProcessor(processor1);
        statistics.addFileProcessor(processor2);

        FileProcessor removed = statistics.removeFileProcessor("Source File");
        FileProcessor query = statistics.getFileProcessor("Text File");
        List processors = statistics.getAllFileProcessors();

        statistics.clearAllFileProcessors();

        // manipulate reporting
        StatsReporting original = statistics.getReporting();

        statistics.setReporting(reporting2);

        // manipulate match all flag
        if (!statistics.isMatchAll()) {
            statistics.setMatchAll(true);
        }
    }

    /**
     * Processes some files and some directories with the file statistics instance. The directory can be processed
     * recursively using a recursive flag. The processing results are aggregated.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcess() throws Exception {
        // create a file statistics instance
        ExtendedFileStatistics statistics = new ExtendedFileStatistics();

        // the file statistics instance should be properly configured here...
        // process some files
        statistics.processFile(new File("test_files/a/p.txt"));
        statistics.processFile(new File("test_files/a/q.java"));

        // process some directories
        statistics.processDirectory(new File("test_files/a"));
        statistics.processDirectory(new File("test_files/a"), true);
    }

    /**
     * Generates report in three ways: as a raw string, to console or to a specified output file. After the report is
     * generated, the reporting implementation can be reset for next use.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGenerateReport() throws Exception {
        // create a file statistics instance
        ExtendedFileStatistics statistics = new ExtendedFileStatistics();

        // the file statistics instance should process some files here...
        // obtain report as a string
        String report = statistics.getReporting().generateReport();

        // output report to console
        statistics.printReport();

        // output report to file
        File file = new File("test_files/print.txt");

        statistics.printReport(file);

        // reset reporting
        statistics.getReporting().reset(new File("test_files/a"));

        file.delete();
    }
}
