/*
 * RegexMatcherTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.RegexMatcher;

import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;


/**
 * Unit tests for RegexMatcher implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class RegexMatcherTest extends TestCase {
    /**
     * The files to match against by RegexMatcher instances.
     */
    private static final String[] FILES = {"test_files/a/p.txt", "test_files/a/q.java", "test_files/b/test.sql"};

    /**
     * The RegexMatcher instances to test against.
     */
    private RegexMatcher[] matchers = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(RegexMatcherTest.class);

        return suite;
    }

    /**
     * Prepares three RegexMatcher instances for testing. matchers[0] is constructed with a single pattern ".\\.txt".
     * matchers[1] is constructed with multiple patterns ".\\.txt", ".\\.java". matchers[2] is constructed with
     * property loaded with patterns ".\\.txt", ".\\.java".
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void setUp() throws Exception {
        ConfigHelper.loadConfig();

        matchers = new RegexMatcher[3];
        matchers[0] = new RegexMatcher(Pattern.compile(".*\\.txt"));
        matchers[1] = new RegexMatcher(Arrays.asList(
                    new Pattern[] {Pattern.compile(".*\\.txt"), Pattern.compile(".*\\.java")}));
        matchers[2] = new RegexMatcher(ConfigHelper.getRootProperty(ConfigHelper.GOOD_REGEX_MATCHER_NAMESPACE));
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor with null pattern. Expects NullPointerException.
     */
    public void testConstructor_NullPattern() {
        try {
            new RegexMatcher((Pattern) null);
            fail("Creates RegexMatcher with null pattern");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with null patterns. Expects NullPointerException.
     */
    public void testConstructor_NullExtensions() {
        try {
            new RegexMatcher((List) null);
            fail("Creates RegexMatcher with null patterns");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty patterns. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyExtensions() {
        try {
            new RegexMatcher(new ArrayList());
            fail("Creates RegexMatcher with empty patterns");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with patterns containing non-Pattern instance. Expects IllegalArgumentException.
     */
    public void testConstructor_NonPatternExtensions() {
        List patterns = new ArrayList();

        // add a non-Pattern element
        patterns.add(new Object());

        try {
            new RegexMatcher(patterns);
            fail("Creates RegexMatcher with patterns containing non-Pattern instance");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null property. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullProperty() throws Exception {
        try {
            new RegexMatcher((Property) null);
            fail("Creates RegexMatcher with null property");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with bad property (no value is present). Expects ConfigurationException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_BadProperty() throws Exception {
        try {
            new RegexMatcher(ConfigHelper.getRootProperty(ConfigHelper.BAD_REGEX_MATCHER_NAMESPACE));
            fail("Creates RegexMatcher with bad property");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * Test of match method with the three files. Expects results for matchers[0]: true, false, false. Expects results
     * for matchers[1]: true, true, false. Expects results for matchers[2]: true, true, false.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMatch() throws Exception {
        File[] files = new File[FILES.length];

        for (int i = 0; i < FILES.length; i++) {
            files[i] = new File(FILES[i]);
        }

        // test with matchers[0] - only matches *.txt file
        assertTrue("Fails to match file", matchers[0].match(files[0]));
        assertFalse("Fails to match file", matchers[0].match(files[1]));
        assertFalse("Fails to match file", matchers[0].match(files[2]));

        // test with matchers[1] - matches *.txt, *.java files
        assertTrue("Fails to match file", matchers[1].match(files[0]));
        assertTrue("Fails to match file", matchers[1].match(files[1]));
        assertFalse("Fails to match file", matchers[1].match(files[2]));

        // test with matchers[2] - matches *.txt, *.java files
        assertTrue("Fails to match file", matchers[2].match(files[0]));
        assertTrue("Fails to match file", matchers[2].match(files[1]));
        assertFalse("Fails to match file", matchers[2].match(files[2]));
    }
}
