/*
 * Metric.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

/**
 * <p>
 * Metric represents a measurement of the statistical result. A metric is identified by its name. Metrics with the same
 * name will be aggregated. This class overrides equals() and hashCode() so that it can be used as map keys.
 * </p>
 * 
 * <p>
 * This class is immutable and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class Metric {
    /**
     * <p>
     * The name of the metric. Acts as an identifier. It is set in constructor and never changed.
     * </p>
     */
    private String name = null;

    /**
     * <p>
     * The display name in the rendered result. If this is used in xml reporting the display name should be well formed
     * element name. It is set in constructor and never changed.
     * </p>
     */
    private String displayName = null;

    /**
     * <p>
     * A description of the metric. This does not make significance in this release. It is set in constructor and
     * never changed.
     * </p>
     */
    private String description = null;

    /**
     * <p>
     * Creates a new Metric instance with the given name, display name and description.
     * </p>
     *
     * @param name the name of the metric
     * @param displayName the display name of the metric
     * @param description the description of the metric
     *
     * @throws NullPointerException if any argument is null
     * @throws IllegalArgumentException if any argument is empty
     */
    public Metric(String name, String displayName, String description) {
        if (name == null) {
            throw new NullPointerException("name is null");
        }
        if (displayName == null) {
            throw new NullPointerException("displayName is null");
        }
        if (description == null) {
            throw new NullPointerException("description is null");
        }
        if (name.trim().length() == 0) {
            throw new IllegalArgumentException("name is empty");
        }
        if (displayName.trim().length() == 0) {
            throw new IllegalArgumentException("displayName is empty");
        }
        if (description.trim().length() == 0) {
            throw new IllegalArgumentException("description is empty");
        }

        this.name = name;
        this.displayName = displayName;
        this.description = description;
    }

    /**
     * <p>
     * Gets the name of the metric. Must be non-null and non-empty.
     * </p>
     *
     * @return the name of the metric
     */
    public String getName() {
        return name;
    }

    /**
     * <p>
     * Gets the display name of the metric. Must be non-null and non-empty.
     * </p>
     *
     * @return the display name of the metric
     */
    public String getDisplayName() {
        return displayName;
    }

    /**
     * <p>
     * Gets the description of the metric. Must be non-null and non-empty.
     * </p>
     *
     * @return the description of the metric
     */
    public String getDescription() {
        return description;
    }

    /**
     * <p>
     * Determines whether the given object is equal to this metric. To be considered equal, the object must be a
     * non-null Metric instance with the same name.
     * </p>
     *
     * @param object the object to check
     *
     * @return whether the object is equal to this metric
     */
    public boolean equals(Object object) {
        if (!(object instanceof Metric)) {
            return false;
        } else {
            return name.equals(((Metric) object).name);
        }
    }

    /**
     * <p>
     * Returns the hash code of this metric. Simply uses the hash code of the name.
     * </p>
     *
     * @return the hash code of this metric
     */
    public int hashCode() {
        return name.hashCode();
    }
}
