/*
 * ExtensionMatcher.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;

import com.topcoder.util.config.Property;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;


/**
 * <p>
 * Concrete implementation of the FileNameMatcher class that matches one or more file extensions. The extensions are
 * matched in a case-insensitive manner.
 * </p>
 * 
 * <p>
 * To follow the reflection contract, this class can be constructed via a Property instance. The supported property is
 * named "extensions" which is potentially multi-valued.
 * </p>
 * 
 * <p>
 * This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class ExtensionMatcher extends FileNameMatcher {
    /**
     * <p>
     * Configuration key that contains multiple file extensions to match. Empty is allowed in order to match a file
     * with no extension.
     * </p>
     */
    public static final String EXTENSIONS_PROPERTY_KEY = "extensions";

    /**
     * <p>
     * A set of file extensions (String) to match. This set is initialized in constructor and is immutable.
     * </p>
     */
    private Set extensions = null;

    /**
     * <p>
     * Constructor with single extension. The extension can be empty to match a file with no extension.
     * </p>
     *
     * @param extension a single file extension to match
     *
     * @throws NullPointerException if extension is null
     */
    public ExtensionMatcher(String extension) {
        if (extension == null) {
            throw new NullPointerException("extension is null");
        }

        // create a Set to hold the single extension
        extensions = new HashSet();
        extensions.add(extension.toLowerCase());
    }

    /**
     * <p>
     * Constructor with multiple extensions. The extension element can be empty String to match a file with no
     * extension. The class will store a copy of the list.
     * </p>
     *
     * @param extensions the multiple extensions to match
     *
     * @throws NullPointerException if extensions is null
     * @throws IllegalArgumentException if extensions is empty or contains non-String instance
     */
    public ExtensionMatcher(List extensions) {
        MatcherUtil.checkListForString(extensions, true);

        // save the normalized extensions
        this.extensions = normalize(extensions);
    }

    /**
     * <p>
     * Constructor that loads extensions from a property. It should look for an "extensions" sub-property and extract
     * multiple values. The property value can be empty to match a file with no extension.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if the "extensions" sub-property is missing or it does not hold any values
     */
    public ExtensionMatcher(Property property) throws ConfigurationException {
        String[] extensions = MatcherUtil.getPropertyValues(property, EXTENSIONS_PROPERTY_KEY, true);

        // save the normalized extensions
        this.extensions = normalize(Arrays.asList(extensions));
    }

    /**
     * <p>
     * Checks whether the filename matches any of the extensions defined in the matcher. The extension is extracted
     * after the last dot in the filename. If the filename does not contain a dot the matching extension must be
     * empty. Matching should be case-insensitive.
     * </p>
     *
     * @param filename the filename to match
     *
     * @return whether the filename matches
     */
    protected boolean match(String filename) {
        int index = filename.lastIndexOf('.');
        String extension = null;

        if (index < 0) {
            // if dot is not present, the extension must be empty
            extension = "";
        } else {
            // otherwise the extension is extracted after the last dot
            extension = filename.substring(index + 1);
        }

        // look for the extension in a case-insensitive way
        return extensions.contains(extension.toLowerCase());
    }

    /**
     * <p>
     * Normalizes the list of extensions for faster matching. It changes the extensions to lower case and stores them
     * in a set to remove duplicates.
     * </p>
     * 
     * <p>
     * This method assumes that the extension list was checked and should be valid.
     * </p>
     *
     * @param extensions the extensions to be normalized
     *
     * @return the normalized extensions
     */
    private Set normalize(List extensions) {
        // store the extensions in a set
        Set result = new HashSet();

        // change the extensions to lower case
        for (Iterator i = extensions.iterator(); i.hasNext();) {
            result.add(((String) i.next()).toLowerCase());
        }
        return result;
    }
}
