/*
 * TextLineCounter.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.processor.linecounter;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.processor.FileProcessorBase;

import com.topcoder.util.config.Property;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;

import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * TextLineCounter counts the number of lines for text files. It supports a single metric of "Line Count".
 * </p>
 * 
 * <p>
 * It delegates configuration to base class without any addition. The implementation still attempts to process the file
 * even if it is not a text file.
 * </p>
 * 
 * <p>
 * This class is stateless while its base class is thread-safe, thus it is thread-safe too.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class TextLineCounter extends FileProcessorBase {
    /**
     * <p>
     * The name of the metric used for this processor.
     * </p>
     */
    public static final String NAME = "Line Count";

    /**
     * <p>
     * The display name of the metric used for this processor.
     * </p>
     */
    public static final String DISPLAY_NAME = "line";

    /**
     * <p>
     * The description of the metric used for this processor.
     * </p>
     */
    private static final String DESCRIPTION = "Number of lines in text files";

    /**
     * <p>
     * The single metric supported by this processor. It is statically initialized and never changed.
     * </p>
     */
    private static final Metric METRIC = new Metric(NAME, DISPLAY_NAME, DESCRIPTION);

    /**
     * <p>
     * Constructor with the given alias and file matcher. Just propagates to super constructor.
     * </p>
     *
     * @param alias the alias for the file processor
     * @param matcher the file matcher associated with the file processor
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty
     */
    public TextLineCounter(String alias, FileMatcher matcher) {
        super(alias, matcher);
    }

    /**
     * <p>
     * This constructor loads configuration from the property. Just propagates to super constructor.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if any error occurs in configuring the processor
     */
    public TextLineCounter(Property property) throws ConfigurationException {
        super(property);
    }

    /**
     * <p>
     * Returns a list containing single Metric composed of the NAME, DISPLAY_NAME and a description.
     * </p>
     *
     * @return a list of metrics the processor supports
     */
    public List getSupportedMetrics() {
        List metrics = new ArrayList();

        // add single metric to the list
        metrics.add(METRIC);

        return metrics;
    }

    /**
     * <p>
     * Counts the number of lines in the given text file. The returned StatsResult will include the line count under
     * the single metric of "Line Count".
     * </p>
     * 
     * <p>
     * An empty file should have a line count of zero.
     * </p>
     *
     * @param file the file to process
     *
     * @return the line count result
     *
     * @throws NullPointerException if file is null
     * @throws IOException if I/O operation fails
     */
    public StatsResult process(File file) throws IOException {
        if (file == null) {
            throw new NullPointerException("file is null");
        }

        // count the number of lines using a BufferedReader
        BufferedReader in = null;
        long count = 0;

        try {
            in = new BufferedReader(new FileReader(file));

            while (in.readLine() != null) {
                count++;
            }
        } finally {
            close(in);
        }

        StatsResult result = new StatsResult();

        // include the line count under the single metric
        result.setStat(METRIC, count);

        return result;
    }

    /**
     * <p>
     * Try to close the reader object if it is not null. Returns silently if any I/O error occurs.
     * </p>
     *
     * @param reader the reader object.
     */
    private static void close(Reader reader) {
        if (reader != null) {
            try {
                reader.close();
            } catch (IOException e) {}
        }
    }
}
