/*
 * FileCategoryMatcherTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.FileCategoryMatcher;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;


/**
 * Unit tests for FileCategoryMatcher implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class FileCategoryMatcherTest extends TestCase {
    /**
     * The binary file for testing.
     */
    private static final String BINARY_FILE = "test_files/a/q.class";

    /**
     * The text file for testing.
     */
    private static final String TEXT_FILE = "test_files/b/test.xml";

    /**
     * The FileCategoryMatcher instance to test against.
     */
    private SimpleFileCategoryMatcher matcher = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(FileCategoryMatcherTest.class);

        return suite;
    }

    /**
     * Prepares a FileCategoryMatcher instance for testing.
     */
    public void setUp() {
        matcher = new SimpleFileCategoryMatcher();
    }

    /**
     * Tests the accessibility of default constructor.
     */
    public void testConstructor_Default() {
        new SimpleFileCategoryMatcher();
    }

    /**
     * Test of isBinary method with null file. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testIsBinary_NullFile() throws Exception {
        try {
            FileCategoryMatcher.isBinary(null);
            fail("Checks if a null file is binary");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of isBinary method with binary file. Verifies if true is returned.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testIsBinary_BinaryFile() throws Exception {
        assertTrue("Returns false with binary file", FileCategoryMatcher.isBinary(new File(BINARY_FILE)));
    }

    /**
     * Test of isBinary method with text file. Verifies if false is returned.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testIsBinary_TextFile() throws Exception {
        assertFalse("Returns true with text file", FileCategoryMatcher.isBinary(new File(TEXT_FILE)));
    }

    /**
     * Test of match method with null file. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMatch_NullFile() throws Exception {
        try {
            matcher.match((File) null);
            fail("Matches a null file");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of match method with non-null file. Verifies if the binary flag of the file was passed to the abstract
     * match() method. Also verifies if it returns the result of the abstract match() method.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMatch_NonNullFile() throws Exception {
        matcher.setMatchResult(true);
        assertTrue("Fails to match file", matcher.match(new File(BINARY_FILE)));
        assertTrue("Fails to match file", matcher.isBinary());
    }

    /**
     * A simple implementation of the FileCategoryMatcher class for testing purpose. Two additional methods are
     * provided to set the matching result of the match() method, and get the binary flag passed to the match()
     * method.
     */
    private class SimpleFileCategoryMatcher extends FileCategoryMatcher {
        /**
         * The matching result of the match() method. It can be changed by its setter method.
         */
        private boolean matchResult = false;

        /**
         * The last binary flag passed to the match() method. It is updated once match() method is called.
         */
        private boolean binary = false;

        /**
         * Default constructor. Does nothing.
         */
        public SimpleFileCategoryMatcher() {
            super();
        }

        /**
         * Sets the matching result of the match() method.
         *
         * @param matchResult the matching result of the match() method.
         */
        public void setMatchResult(boolean matchResult) {
            this.matchResult = matchResult;
        }

        /**
         * Returns the last binary flag passed to the match() method. It is false if match() method is not called.
         *
         * @return the last binary flag passed to the match() method.
         */
        public boolean isBinary() {
            return binary;
        }

        /**
         * <p>
         * Determines whether the file matches based on the given file category. The result depends on the status set
         * in the setMatchResult() method. In addition, it saves the passed binary flag.
         * </p>
         *
         * @param isBinary whether the file is binary
         *
         * @return whether the file category matches
         */
        protected boolean match(boolean isBinary) {
            binary = isBinary;
            return matchResult;
        }
    }
}
