/*
 * FileProcessorBaseTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.matcher.BinaryFileMatcher;
import com.topcoder.file.statistics.matcher.TextFileMatcher;
import com.topcoder.file.statistics.processor.FileProcessorBase;

import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import java.util.List;


/**
 * Unit tests for FileProcessorBase implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class FileProcessorBaseTest extends TestCase {
    /**
     * The alias of the processors for testing.
     */
    private static final String ALIAS = "alias";

    /**
     * The FileProcessorBase instances to test against.
     */
    private FileProcessorBase[] processors = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(FileProcessorBaseTest.class);

        return suite;
    }

    /**
     * Prepares two FileProcessorBase instances for testing. processors[0] is constructed with the alias "alias" and an
     * BinaryFileMatcher. processors[1] is constructed with property loaded with alias "alias" and a TextFileMatcher.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void setUp() throws Exception {
        ConfigHelper.loadConfig();

        processors = new SimpleFileProcessorBase[2];
        processors[0] = new SimpleFileProcessorBase(ALIAS, new BinaryFileMatcher());
        processors[1] = new SimpleFileProcessorBase(ConfigHelper.getRootProperty(
                    ConfigHelper.GOOD_FILE_PROCESSOR_BASE_NAMESPACE));
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor with null alias. Expects NullPointerException.
     */
    public void testConstructor_NullAlias() {
        try {
            new SimpleFileProcessorBase(null, new BinaryFileMatcher());
            fail("Creates FileProcessorBase with null alias");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty alias. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyAlias() {
        try {
            new SimpleFileProcessorBase("", new BinaryFileMatcher());
            fail("Creates FileProcessorBase with empty alias");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null matcher. Expects NullPointerException.
     */
    public void testConstructor_NullMatcher() {
        try {
            new SimpleFileProcessorBase(ALIAS, null);
            fail("Creates FileProcessorBase with null matcher");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with null property. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullProperty() throws Exception {
        try {
            new SimpleFileProcessorBase(null);
            fail("Creates FileProcessorBase with null property");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with bad property (no alias property). Expects ConfigurationException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_BadProperty() throws Exception {
        try {
            new SimpleFileProcessorBase(ConfigHelper.getRootProperty(ConfigHelper.BAD_FILE_PROCESSOR_BASE_NAMESPACE));
            fail("Creates FileProcessorBase with bad property");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * Test of getAlias method. Verifies if it returns "alias" for both processors[0] and processors[1].
     */
    public void testGetAlias() {
        assertEquals("Returns an incorrect alias", ALIAS, processors[0].getAlias());
        assertEquals("Returns an incorrect alias", ALIAS, processors[1].getAlias());
    }

    /**
     * Test of getFileMatcher method. Verifies if it returns an BinaryFileMatcher for processors[0]. Verifies if it
     * returns a TextFileMatcher for processors[1].
     */
    public void testGetFileMatcher() {
        assertTrue("Returns an incorrect matcher", processors[0].getFileMatcher() instanceof BinaryFileMatcher);
        assertTrue("Returns an incorrect matcher", processors[1].getFileMatcher() instanceof TextFileMatcher);
    }

    /**
     * A simple implementation of the FileProcessorBase class for testing purpose. The constructors just propagate to
     * the super constructors. The getSupportedMetrics() and process() methods do nothing.
     */
    private class SimpleFileProcessorBase extends FileProcessorBase {
        /**
         * Constructor with the given alias and file matcher. Just propagates to the super constructor.
         *
         * @param alias the alias for the file processor
         * @param matcher the file matcher associated with the file processor
         *
         * @throws NullPointerException if alias or matcher is null
         * @throws IllegalArgumentException if alias is empty
         */
        public SimpleFileProcessorBase(String alias, FileMatcher matcher) {
            super(alias, matcher);
        }

        /**
         * This constructor loads configuration from the property. Just propagates to the super constructor.
         *
         * @param property the property to configure from
         *
         * @throws NullPointerException if property is null
         * @throws ConfigurationException if any error occurs in configuring the processor
         */
        public SimpleFileProcessorBase(Property property)
            throws ConfigurationException
        {
            super(property);
        }

        /**
         * <p>
         * Gets a list of all the supported metrics by this processor.
         * </p>
         *
         * @return always null
         */
        public List getSupportedMetrics() {
            return null;
        }

        /**
         * <p>
         * Processes the file and generates the result.
         * </p>
         *
         * @param file the file to process
         *
         * @return always null
         */
        public StatsResult process(File file) {
            return null;
        }
    }
}
