/*
 * FileStatisticsTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.TextFileMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.BasicStatsReporting;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import java.util.ArrayList;
import java.util.List;


/**
 * Unit tests for FileStatistics implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class FileStatisticsTest extends TestCase {
    /**
     * The path of the base directory used in the reporting.
     */
    private static final String BASE_DIRECTORY = "test_files/";

    /**
     * The path of the file to be processed.
     */
    private static final String FILE = "test_files/a/q.java";

    /**
     * The path of the expected report when matchall flag is false.
     */
    private static final String MATCHALL_FALSE_REPORT_FILE = "test_files/matchall_false.txt";

    /**
     * The path of the expected report when matchall flag is true.
     */
    private static final String MATCHALL_TRUE_REPORT_FILE = "test_files/matchall_true.txt";

    /**
     * The FileStatistics instance to test against.
     */
    private FileStatistics stats = null;

    /**
     * The file processor to be added. (TextLineCounter)
     */
    private FileProcessor processor = null;

    /**
     * The file processor to be added. (CStyleLineCounter)
     */
    private FileProcessor processor2 = null;

    /**
     * The matcher associated with the file processors.
     */
    private FileMatcher matcher = null;

    /**
     * The reporting implementation of the FileStatistics instance. (BasicStatsReporting)
     */
    private StatsReporting reporting = null;

    /**
     * The file to be processed.
     */
    private File file = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(FileStatisticsTest.class);

        return suite;
    }

    /**
     * Prepares a FileStatistics instance for testing. The reporting is set to BasicStatsReporting and the matchall
     * flag is set to false. Also prepares the associated matcher and processors.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void setUp() throws Exception {
        ConfigHelper.loadConfig();

        matcher = new TextFileMatcher();
        processor = new TextLineCounter("Text File", matcher);
        processor2 = new CStyleLineCounter("Source File", matcher);
        reporting = new BasicStatsReporting();
        reporting.reset(new File(BASE_DIRECTORY));
        file = new File(FILE);

        stats = new FileStatistics();
        stats.setReporting(reporting);
        stats.setMatchAll(false);
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor with null namespace. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullNamespace() throws Exception {
        try {
            new FileStatistics(null);
            fail("Creates FileStatistics with null namespace");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty namespace. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_EmptyNamespace() throws Exception {
        try {
            new FileStatistics("");
            fail("Creates FileStatistics with empty namespace");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with bad namespace (no processors is present). Expects ConfigurationException.
     */
    public void testConstructor_BadNamespace() {
        try {
            new FileStatistics(ConfigHelper.BAD_FILE_STATISTICS_NAMESPACE);
            fail("Creates FileStatistics with bad namespace");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * Test of constructor with good namespace. Verifies if two processors TextLineCounter and CStyleLineCounter are
     * loaded, the reporting is of type BasicStatsReporting, and the matchall flag is set to true.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_GoodNamespace() throws Exception {
        stats = new FileStatistics(ConfigHelper.GOOD_FILE_STATISTICS_NAMESPACE);

        // verify the processors
        List processors = stats.getAllFileProcessors();

        assertEquals("Fails to load processors", 2, processors.size());
        assertTrue("Fails to load processors", processors.get(0) instanceof TextLineCounter);
        assertTrue("Fails to load processors", processors.get(1) instanceof CStyleLineCounter);

        // verify the reporting
        assertTrue("Fails to load reporting", stats.getReporting() instanceof BasicStatsReporting);

        // verify the matchall flag
        assertTrue("Fails to load matchall flag", stats.isMatchAll());
    }

    /**
     * Test of addFileProcessor method with null processor. Expects NullPointerException.
     */
    public void testAddFileProcessor_NullProcessor() {
        try {
            stats.addFileProcessor(null);
            fail("Adds a null processor");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of addFileProcessor method with non-existing processor. Verifies if the processor was added to the end of
     * the list.
     */
    public void testAddFileProcessor_NonExistProcessor() {
        stats.addFileProcessor(processor2);
        stats.addFileProcessor(processor);

        List processors = stats.getAllFileProcessors();

        assertEquals("Fails to add processor", 2, processors.size());
        assertEquals("Fails to add processor", processor, processors.get(1));
    }

    /**
     * Test of addFileProcessor method with existing processor. Verifies if the processor was added to the end of the
     * list, and the old processor was removed.
     */
    public void testAddFileProcessor_ExistProcessor() {
        // make sure an existing processor
        stats.addFileProcessor(processor);
        stats.addFileProcessor(processor2);

        stats.addFileProcessor(processor);

        List processors = stats.getAllFileProcessors();

        assertEquals("Fails to add processor", 2, processors.size());
        assertEquals("Fails to add processor", processor, processors.get(1));
    }

    /**
     * Test of removeFileProcessor method with null alias. Expects NullPointerException.
     */
    public void testRemoveFileProcessor_NullAlias() {
        try {
            stats.removeFileProcessor(null);
            fail("Removes processor with null alias");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of removeFileProcessor method with empty alias. Expects IllegalArgumentException.
     */
    public void testRemoveFileProcessor_EmptyAlias() {
        try {
            stats.removeFileProcessor("");
            fail("Removes processor with empty alias");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of removeFileProcessor method with non-existing alias. Verifies if it returns null.
     */
    public void testRemoveFileProcessor_NonExistAlias() {
        assertNull("Fails to remove processor", stats.removeFileProcessor("non-exist"));
    }

    /**
     * Test of removeFileProcessor method with existing alias. Verifies if it returns the removed processor. Also
     * verifies if the processor was removed from the list.
     */
    public void testRemoveFileProcessor_ExistAlias() {
        // make sure an existing processor
        stats.addFileProcessor(processor);

        String alias = processor.getAlias();

        assertEquals("Fails to remove processor", processor, stats.removeFileProcessor(alias));
        assertNull("Fails to remove processor", stats.getFileProcessor(alias));
    }

    /**
     * Test of getFileProcessor method with null alias. Expects NullPointerException.
     */
    public void testGetFileProcessor_NullAlias() {
        try {
            stats.getFileProcessor(null);
            fail("Gets processor with null alias");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of getFileProcessor method with empty alias. Expects IllegalArgumentException.
     */
    public void testGetFileProcessor_EmptyAlias() {
        try {
            stats.getFileProcessor("");
            fail("Gets processor with empty alias");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getFileProcessor method with non-existing alias. Verifies if it returns null.
     */
    public void testGetFileProcessor_NonExistAlias() {
        assertNull("Returns an incorrect processor", stats.getFileProcessor("non-exist"));
    }

    /**
     * Test of getFileProcessor method with existing alias. Verifies if it returns the processor with the alias.
     */
    public void testGetFileProcessor_ExistAlias() {
        // make sure an existing processor
        stats.addFileProcessor(processor);

        assertEquals("Returns an incorrect processor", processor, stats.getFileProcessor(processor.getAlias()));
    }

    /**
     * Test of getAllFileProcessors method with empty processors. Verifies if it returns an empty list.
     */
    public void testGetAllFileProcessors_Empty() {
        assertTrue("Returns an incorrect processor list", stats.getAllFileProcessors().isEmpty());
    }

    /**
     * Test of getAllFileProcessors method with non-empty processors. Verifies if it returns a list containing all the
     * processors.
     */
    public void testGetAllFileProcessors_NonEmpty() {
        List processors = new ArrayList();

        // make sure non-empty processors
        for (int i = 0; i < 3; i++) {
            FileProcessor processor = new TextLineCounter("alias" + i, matcher);

            processors.add(processor);
            stats.addFileProcessor(processor);
        }

        assertEquals("Returns an incorrect processor list", processors, stats.getAllFileProcessors());
    }

    /**
     * Test of clearAllFileProcessors method with empty processors. Verifies if the list is still empty after clearing
     * it.
     */
    public void testClearAllFileProcessors_Empty() {
        stats.clearAllFileProcessors();
        assertTrue("Fails to clear processors", stats.getAllFileProcessors().isEmpty());
    }

    /**
     * Test of clearAllFileProcessors method with non-empty processors. Verifies if the list is empty after clearing
     * it.
     */
    public void testClearAllFileProcessors_NonEmpty() {
        // make sure non-empty processors
        for (int i = 0; i < 3; i++) {
            stats.addFileProcessor(new TextLineCounter("alias" + i, matcher));
        }

        stats.clearAllFileProcessors();
        assertTrue("Fails to clear processors", stats.getAllFileProcessors().isEmpty());
    }

    /**
     * Test of getReporting and setReporting methods with non-null reporting. Verifies it by getting the reporting back
     * after setting it.
     */
    public void testGetSetReporting_NonNullReporting() {
        stats.setReporting(reporting);
        assertEquals("Returns an incorrect reporting", reporting, stats.getReporting());
    }

    /**
     * Test of setReporting method with null reporting. Expects NullPointerException.
     */
    public void testSetReporting_NullReporting() {
        try {
            stats.setReporting(null);
            fail("Sets a null reporting");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of isMatchAll and setMatchAll methods. Verifies it by getting the matchall flag back after setting it.
     */
    public void testIsSetMatchAll() {
        stats.setMatchAll(true);
        assertTrue("Returns an incorrect matchall flag", stats.isMatchAll());
    }

    /**
     * Test of processFile method with null file. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcessFile_NullFile() throws Exception {
        try {
            stats.processFile(null);
            fail("Processes a null file");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of processFile method with false matchall flag. First, add the file processors to the stats. Verifies if
     * the file was processed by the first matching processor TextLineCounter from the generated report.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcessFile_FalseMatchAll() throws Exception {
        stats.addFileProcessor(processor);
        stats.addFileProcessor(processor2);
        stats.setMatchAll(false);

        stats.processFile(file);

        assertTrue("Fails to process the file",
            TestHelper.verifyReport(new File(MATCHALL_FALSE_REPORT_FILE), reporting.generateReport()));
    }

    /**
     * Test of processFile method with true matchall flag. First, add the file processors to the stats. Verifies if the
     * file was processed by the all matching processors TextLineCounter and CStyleLineCounter from the generated
     * report.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcessFile_TrueMatchAll() throws Exception {
        stats.addFileProcessor(processor);
        stats.addFileProcessor(processor2);
        stats.setMatchAll(true);

        stats.processFile(file);

        assertTrue("Fails to process the file",
            TestHelper.verifyReport(new File(MATCHALL_TRUE_REPORT_FILE), reporting.generateReport()));
    }
}
