/*
 * StatsCollectionTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.reporting.StatsCollection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.Arrays;


/**
 * Unit tests for StatsCollection implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class StatsCollectionTest extends TestCase {
    /**
     * The key of the StatsResult in the StatsCollection instance.
     */
    private static final String KEY = "key";

    /**
     * The StatsCollection instance to test against.
     */
    private StatsCollection stats = null;

    /**
     * The StatsResult to be added to the StatsCollection instance.
     */
    private StatsResult result = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(StatsCollectionTest.class);

        return suite;
    }

    /**
     * Prepares StatsCollection and StatsResult instances for testing. The StatsResult will be added into the
     * StatsCollection later.
     */
    public void setUp() {
        stats = new StatsCollection();
        result = new StatsResult();
        result.setStat(new Metric("name", "displayName", "description"), 100);
    }

    /**
     * Tests the accessibility of constructor.
     */
    public void testConstructor() {
        new StatsCollection();
    }

    /**
     * Test of add method with null key. Expects NullPointerException.
     */
    public void testAdd_NullKey() {
        try {
            stats.add(null, result);
            fail("Adds stats with null key");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of add method with empty key. Expects IllegalArgumentException.
     */
    public void testAdd_EmptyKey() {
        try {
            stats.add("", result);
            fail("Adds stats with empty key");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of add method with null stats. Expects NullPointerException.
     */
    public void testAdd_NullStats() {
        try {
            stats.add(KEY, null);
            fail("Adds stats with null stats");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of add method with non-existing key. Verifies if the stats was added into the collection. Also verifies if
     * a clone of the result was saved.
     */
    public void testAdd_NonExistKey() {
        StatsResult newResult = new StatsResult(result);

        stats.add(KEY, newResult);
        assertTrue("Fails to add the stats", TestHelper.equals(result, stats.get(KEY)));

        newResult.aggregate(result);
        assertTrue("Fails to clone the stats", TestHelper.equals(result, stats.get(KEY)));
    }

    /**
     * Test of add method with existing key. Verifies if the stats was aggregated in the collection.
     */
    public void testAdd_ExistKey() {
        // make sure the key exists
        stats.add(KEY, result);

        stats.add(KEY, result);

        // aggregate the results for checking
        StatsResult newResult = new StatsResult(result);

        newResult.aggregate(result);

        assertTrue("Fails to aggregate the stats", TestHelper.equals(newResult, stats.get(KEY)));
    }

    /**
     * Test of get method with null key. Expects NullPointerException.
     */
    public void testGet_NullKey() {
        try {
            stats.get(null);
            fail("Gets stats with null key");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of get method with empty key. Expects IllegalArgumentException.
     */
    public void testGet_EmptyKey() {
        try {
            stats.get("");
            fail("Gets stats with empty key");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of get method with non-existing key. Verifies if null is returned.
     */
    public void testGet_NonExistKey() {
        assertNull("Returns an incorrect stats", stats.get("non-exist"));
    }

    /**
     * Test of get method with existing key. Verifies if the cloned result is returned.
     */
    public void testGet_ExistKey() {
        // make sure the key exists
        stats.add(KEY, result);

        StatsResult newResult = stats.get(KEY);

        assertTrue("Returns an incorrect stats", TestHelper.equals(result, newResult));

        newResult.aggregate(result);

        assertTrue("Fails to clone the returned stats", TestHelper.equals(result, stats.get(KEY)));
    }

    /**
     * Test of getAllKeys method with empty keys. Verifies if an empty list is returned.
     */
    public void testGetAllKeys_Empty() {
        assertTrue("Returns an incorrect key list", stats.getAllKeys().isEmpty());
    }

    /**
     * Test of getAllKeys method with non-empty keys. Verifies if the keys are sorted in alphabetical order (case
     * insensitive) in the returned list.
     */
    public void testGetAllKeys_NonEmpty() {
        String[] keys = {"image", "Source", "text"};

        // make sure non-empty keys
        for (int i = keys.length - 1; i >= 0; i--) {
            stats.add(keys[i], new StatsResult());
        }

        assertEquals("Returns an incorrect key list", Arrays.asList(keys), stats.getAllKeys());
    }

    /**
     * Test of clear method with empty keys. Verifies if the key list is still empty after clearing it.
     */
    public void testClear_Empty() {
        stats.clear();
        assertTrue("Fails to clear the keys", stats.getAllKeys().isEmpty());
    }

    /**
     * Test of clear method with non-empty keys. Verifies if the key list is empty after clearing it.
     */
    public void testClear_NonEmpty() {
        // make sure non-empty keys
        for (int i = 0; i < 3; i++) {
            stats.add("key" + i, new StatsResult());
        }

        stats.clear();
        assertTrue("Fails to clear the keys", stats.getAllKeys().isEmpty());
    }
}
